"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const diff_classifier_1 = require("./spec-differ/diff-classifier");
const diff_finder_1 = require("./spec-differ/diff-finder");
const spec_loader_1 = require("./spec-differ/spec-loader");
const spec_finder_1 = require("./spec-finder");
const spec_parser_1 = require("./spec-parser");
class SpecDiffer {
    static diffSpecs({ sourceSpec, destinationSpec }) {
        return __awaiter(this, void 0, void 0, function* () {
            const loadedSourceSpec = spec_loader_1.SpecLoader.load(sourceSpec);
            const loadedDestinationSpec = spec_loader_1.SpecLoader.load(destinationSpec);
            const parsedSourceSpec = spec_parser_1.specParser.parse(loadedSourceSpec);
            const parsedDestinationSpec = spec_parser_1.specParser.parse(loadedDestinationSpec);
            const resultsByType = yield this.diffSourceAndDestinationParsedSpecs(parsedSourceSpec, parsedDestinationSpec);
            const sourceSpecDetails = {
                format: parsedSourceSpec.format,
                location: sourceSpec.location
            };
            const destinationSpecDetails = {
                format: parsedDestinationSpec.format,
                location: destinationSpec.location
            };
            return this.generateDiffOutcome(resultsByType, sourceSpecDetails, destinationSpecDetails);
        });
    }
    static newDiffSpecs({ sourceSpec, destinationSpec }) {
        return __awaiter(this, void 0, void 0, function* () {
            const loadedSourceSpec = spec_loader_1.SpecLoader.load(sourceSpec);
            const loadedDestinationSpec = spec_loader_1.SpecLoader.load(destinationSpec);
            const parsedSourceSpec = spec_parser_1.specParser.parse(loadedSourceSpec);
            const parsedDestinationSpec = spec_parser_1.specParser.parse(loadedDestinationSpec);
            const differences = yield diff_finder_1.DiffFinder.findDifferences({
                destinationSpec: parsedDestinationSpec,
                sourceSpec: parsedSourceSpec
            });
            const classifiedDifferences = diff_classifier_1.DiffClassifier.classifyDifferences(differences);
            const sourceSpecDetails = {
                format: parsedSourceSpec.format,
                location: sourceSpec.location
            };
            const destinationSpecDetails = {
                format: parsedDestinationSpec.format,
                location: destinationSpec.location
            };
            return this.newGenerateDiffOutcome(classifiedDifferences, sourceSpecDetails, destinationSpecDetails);
        });
    }
    static newGenerateDiffOutcome(classifiedDifferences, sourceSpecDetails, destinationSpecDetails) {
        const breakingDifferencesFound = classifiedDifferences.breakingDifferences.length > 0;
        if (breakingDifferencesFound) {
            return {
                breakingDifferences: classifiedDifferences.breakingDifferences,
                breakingDifferencesFound,
                destinationSpecDetails,
                nonBreakingDifferences: classifiedDifferences.nonBreakingDifferences,
                sourceSpecDetails,
                unclassifiedDifferences: classifiedDifferences.unclassifiedDifferences
            };
        }
        else {
            return {
                breakingDifferencesFound,
                destinationSpecDetails,
                nonBreakingDifferences: classifiedDifferences.nonBreakingDifferences,
                sourceSpecDetails,
                unclassifiedDifferences: classifiedDifferences.unclassifiedDifferences
            };
        }
    }
    static generateDiffOutcome(resultsByType, sourceSpecDetails, destinationSpecDetails) {
        const breakingDifferencesFound = resultsByType.breakingDifferences.length > 0;
        if (breakingDifferencesFound) {
            return {
                breakingDifferences: resultsByType.breakingDifferences,
                breakingDifferencesFound,
                destinationSpecDetails,
                nonBreakingDifferences: resultsByType.nonBreakingDifferences,
                sourceSpecDetails,
                unclassifiedDifferences: resultsByType.unclassifiedDifferences
            };
        }
        else {
            return {
                breakingDifferencesFound,
                destinationSpecDetails,
                nonBreakingDifferences: resultsByType.nonBreakingDifferences,
                sourceSpecDetails,
                unclassifiedDifferences: resultsByType.unclassifiedDifferences
            };
        }
    }
    static diffSourceAndDestinationParsedSpecs(source, destination) {
        return __awaiter(this, void 0, void 0, function* () {
            const allResults = yield spec_finder_1.specFinder.diff(source, destination);
            return {
                breakingDifferences: allResults.filter((result) => result.type === 'breaking'),
                nonBreakingDifferences: allResults.filter((result) => result.type === 'non-breaking'),
                unclassifiedDifferences: allResults.filter((result) => result.type === 'unclassified')
            };
        });
    }
}
exports.SpecDiffer = SpecDiffer;
