"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const yaml = require("js-yaml");
const VError = require("verror");
class SpecLoader {
    static load(specOption) {
        return SpecLoader.parseSpec(specOption.location, specOption.content);
    }
    static parseSpec(location, content) {
        try {
            return SpecLoader.parseJson(location, content);
        }
        catch (parseJsonError) {
            return SpecLoader.parseYaml(location, content);
        }
    }
    static parseJson(location, content) {
        try {
            return JSON.parse(content);
        }
        catch (error) {
            throw new VError(`ERROR: unable to parse ${location} as a JSON or YAML file`);
        }
    }
    static parseYaml(location, content) {
        let parsedYaml = null;
        try {
            parsedYaml = yaml.safeLoad(content);
        }
        catch (error) {
            throw new VError(`ERROR: unable to parse ${location} as a JSON or YAML file`);
        }
        if (!parsedYaml) {
            throw new VError(`ERROR: unable to parse ${location} as a JSON or YAML file`);
        }
        return parsedYaml;
    }
}
exports.SpecLoader = SpecLoader;
